(* 
   SNU 4541.664A HW2
*)   

{
    open Parser
    open Error

    val debug_tag = false
    fun verbose s =  if debug_tag then (print_string s; print_newline())

    val comment_depth = ref 0

    val keyword_tbl = Hashtbl.create 9

    val _ = List.iter (fn (keyword, tok) => Hashtbl.add keyword_tbl keyword tok)
                   [
                    ("skip", SKIP),

                    ("readint", READINT),
                    ("print", PRINT),

                    ("while", WHILE),
                    ("if", IF),
                    ("then",THEN),
                    ("else",ELSE),
                    ("local", LOCAL),
                    ("in", IN),
                    ("end", END),
                    ("do", DO)
                  ]
     
    fun s2int "" = raise Lex_err("illegal number token", get_ln()) 
	| s2int s = if ('~' = String.get s 0) then
                   - (int_of_string(String.sub s 1 ((String.length s)-1)))
                   else int_of_string s
} 

val blank = [' ' '\t']+
    (* KS X 1001 a.k.a. KS C 5601 or euc-kr *)
val id = ['a'-'z' 'A'-'Z'](['a'-'z' 'A'-'Z' '\'' '0'-'9' '_'])*
val number = ['0'-'9']+|'~'['0'-'9']+
val charstring = '"' [^ '"' '\n']* '"'

rule start =
 parse blank { start lexbuf }
     | "\r\n"     { incr_ln (); start lexbuf}
     | '\n'       { incr_ln (); start lexbuf}
     | "//"[^'\n']* '\n' { incr_ln(); start lexbuf }
     | number { NUM (s2int(Lexing.lexeme lexbuf)) }
     | id { let val id = Lexing.lexeme lexbuf
            in Hashtbl.find keyword_tbl id
               handle _ => ID id
            end}
     | eof { verbose "eof"; EOF}
     | "(*" { comment_depth :=1;
              comment lexbuf;
              start lexbuf }
     | ";" { verbose ";"; SEMICOLON}
     | ":=" { verbose ":="; COLONEQ}
     | "=" {verbose "="; EQUAL}
     | "+" {verbose "+"; PLUS}
     | "-" {verbose "-"; MINUS}
     | "*" {verbose "-"; TIMES}
     | "<" {verbose "-"; LT}
     | "(" { verbose "("; LP}
     | ")" { verbose ")"; RP}
     | _ {raise Lex_err("illical token "^(Lexing.lexeme lexbuf), get_ln())} 

and comment = parse
     "(*" {comment_depth := !comment_depth+1; comment lexbuf}
   | "*)" {comment_depth := !comment_depth-1;
           if !comment_depth > 0 then comment lexbuf }
   | eof {raise Lex_err("Eof within comment",get_ln())}
   | '\n' {incr_ln(); comment lexbuf}
   | "\r\n" {incr_ln(); comment lexbuf}
   | _   {comment lexbuf}
